local addon, db = ...

db.init_reagents = function ()
	if db.SV.DONT_BUY_REAGENTS then return end

	local reagents = {}

	local check_glyphs = false
	local glyph_is_missing
	local update_reagents

	local class = select(2, UnitClass"player")
	if class == "DRUID" then
		check_glyphs = true -- Glyph of Unburdened Rebirth
		update_reagents = function (level)
			-- Gift of the Wild reagent
			if level >= 80 then
				reagents[44605] = 1
			elseif level >= 70 then
				reagents[22148] = 1
			elseif level >= 60 then
				reagents[17026] = 1
			elseif level >= 50 then
				reagents[17021] = 1
			end
			-- Rebirth
			if glyph_is_missing(58161) then
				if level >= 79 then
					reagents[44614] = 1
				elseif level >= 69 then
					reagents[22147] = 1
				elseif level >= 60 then
					reagents[17038] = 1
				elseif level >= 50 then
					reagents[17037] = 1
				elseif level >= 40 then
					reagents[17036] = 1
				elseif level >= 30 then
					reagents[17035] = 1
				elseif level >= 20 then
					reagents[17034] = 1
				end
			end
		end
	elseif class == "MAGE" then
		update_reagents = function (level)
			-- Rune of Teleportation
			if level >= 20 then
				reagents[17031] = 1
			end
			-- Rune of Portals
			if level >= 35 then
				reagents[17032] = 1
			end
			-- Arcane Powder
			if level >= 56 then
				reagents[17020] = 1
			end
		end
	elseif class == "PALADIN" then
		update_reagents = function (level)
			-- Symbol of Divinity
			if level >= 30 then
				reagents[17033] = 1
			end
			-- Symbol of Kings
			if level >= 52 then
				reagents[21177] = 1
			end
		end
	elseif class == "PRIEST" then
		update_reagents = function (level)
			-- Candles reagents
			if level >= 80 then
				reagents[44615] = 2
			elseif level >= 77 then
				reagents[44615] = 1
				reagents[17029] = 1
			elseif level >= 60 then
				reagents[17029] = 2
			elseif level >= 56 then
				reagents[17029] = 1
				reagents[17028] = 1
			elseif level >= 48 then
				reagents[17028] = 1
			end
		end
	elseif class == "SHAMAN" then
		check_glyphs = true -- Glyph of Renewed Life
		update_reagents = function (level)
			-- Ankh
			if level >= 30 and glyph_is_missing(58263) then
				reagents[17030] = 1
			end
		end
	elseif class == "ROGUE" then
		update_reagents = function (level)
			-- Crippling Poison
			if level >= 20 then
				reagents[3775] = 1
			end
			-- Mind-numbing Poison
			if level >= 24 then
				reagents[5237] = 1
			end
			-- Wound Poison
			if level >= 78 then
				reagents[43235] = 1
			elseif level >= 72 then
				reagents[43234] = 1
			elseif level >= 64 then
				reagents[22055] = 1
			elseif level >= 56 then
				reagents[10922] = 1
			elseif level >= 48 then
				reagents[10921] = 1
			elseif level >= 40 then
				reagents[10920] = 1
			elseif level >= 32 then
				reagents[10918] = 1
			end
			-- Instant Poison
			if level >= 79 then
				reagents[43231] = 1
			elseif level >= 73 then
				reagents[43230] = 1
			elseif level >= 68 then
				reagents[21927] = 1
			elseif level >= 60 then
				reagents[8928] = 1
			elseif level >= 52 then
				reagents[8927] = 1
			elseif level >= 44 then
				reagents[8926] = 1
			elseif level >= 36 then
				reagents[6950] = 1
			elseif level >= 28 then
				reagents[6949] = 1
			elseif level >= 20 then
				reagents[6947] = 1
			end
			-- Deadly Poison
			if level >= 80 then
				reagents[43233] = 1
			elseif level >= 76 then
				reagents[43232] = 1
			elseif level >= 70 then
				reagents[22054] = 1
			elseif level >= 62 then
				reagents[22053] = 1
			elseif level >= 60 then
				reagents[20844] = 1
			elseif level >= 54 then
				reagents[8985] = 1
			elseif level >= 46 then
				reagents[8984] = 1
			elseif level >= 38 then
				reagents[2893] = 1
			elseif level >= 30 then
				reagents[2892] = 1
			end
			-- Anesthetic Poison
			if level >= 77 then
				reagents[43237] = 1
			elseif level >= 68 then
				reagents[21835] = 1
			end
		end
	elseif class == "DEATHKNIGHT" then
		check_glyphs = true -- Glyph of Raise Dead
		update_reagents = function (level)
			-- Corpse Dust
			if glyph_is_missing(60200) then
				reagents[37201] = 1
			end
		end
	else
		update_reagents = nil
	end

	if not update_reagents then return end

	db.Register(function (self)
		for item, stacks in pairs(reagents) do
			local link = select(2, GetItemInfo(item))
			if link then
				local maxStack = select(8, GetItemInfo(item))
				local count = stacks * maxStack
				local have = GetItemCount(link)
				local tobuy = count - have
				if tobuy > 0 then
					for i = 1, GetMerchantNumItems() do
						if GetMerchantItemLink(i) == link then
							local quantity = select(4, GetMerchantItemInfo(i))
							while tobuy > 0 do
								local c = math.min(tobuy, maxStack)
								c = math.floor(c / quantity)
								if c == 0 then break end
								BuyMerchantItem(i, c)
								tobuy = tobuy - (c * quantity)
							end
							break
						end
					end
				end
			end
		end
		return true
	end, "MERCHANT_SHOW")

	db.Register(function (self)
		update_reagents(UnitLevel"player")
	end, "PLAYER_ENTERING_WORLD")

	db.Register(function (self, _, level)
		reagents = {}
		update_reagents(tonumber(level))
		return true
	end, "PLAYER_LEVEL_UP")

	if not check_glyphs then return end

	local function has_glyph(spellId, talentGroup)
		for socket = 1, 6 do
			local enabled, _, spell = GetGlyphSocketInfo(socket, talentGroup)
			if spell == spellId then return true end
		end
	end

	if db.SV.GLYPH_ONCE_IS_ENOUGH then
		glyph_is_missing = function (spellId)
			for talentGroup = 1, GetNumTalentGroups() do
				if has_glyph(spellId, talentGroup) then
					return false
				end
			end
			return true
		end
	else
		glyph_is_missing = function (spellId)
			for talentGroup = 1, GetNumTalentGroups() do
				if not has_glyph(spellId, talentGroup) then
					return true
				end
			end
		end
	end

	db.Register(function (self)
		reagents = {}
		update_reagents(UnitLevel"player")
		return true
	end, "GLYPH_ADDED", "GLYPH_REMOVED", "GLYPH_UPDATED")
end
